import logging

import requests

from .errors import BadRequest, Unauthorised, Forbidden, TooManyRequests, NotFound
from .urls import URLs


#Fetches response from the relevant endpoints and applies parameters to the request as needed are needed
class API(URLs):

    def __init__(self):
        super().__init__()

    #Method handles error codes from the API
    def _response_code_error_type(self, resp):
        if resp.status_code == 400:
            raise BadRequest(response=resp)
        if resp.status_code == 401:
            raise Unauthorised(response=resp)
        if resp.status_code == 403:
            raise Forbidden(response=resp)
        if resp.status_code == 404:
            raise NotFound(response=resp)
        if resp.status_code == 429:
            raise TooManyRequests(response=resp)


    def make_request(self, endpoint: str=None, params: dict={}) -> str:

        logging.debug(msg=f'Fetching resp from endpoint: {endpoint} with params: {params}')

        try:
            response = requests.get(url=endpoint, params=params)

            if response.status_code != 200:
                self._response_code_error_type(response)

            logging.debug(msg=f'Returning resp from endpoint: {endpoint} with params: {params}')
            return response

        except Exception as err:
            logging.error(msg=err, exc_info=True)
            raise



from .api import API

class Wrapper(API):

    def __init__(self):
        super().__init__()

    #Enter the name of a given endpoint as arguement and recieve the endpoint as return
    def _fetch_endpoint(self, endpoint_name: str=None) ->str:

        if endpoint_name == 'legislation':
            return self._legislation_url()
        if endpoint_name == 'debates':
            return self._debates_url()
        if endpoint_name == 'constituencies':
            return self._constituencies_url()
        if endpoint_name == 'parties':
            return self._parties_url()
        if endpoint_name == 'divisions':
            return self._divisions_url()
        if endpoint_name == 'questions':
            return self._questions_url()
        if endpoint_name == 'houses':
            return self._houses_url()
        if endpoint_name == 'members':
            return self._members_url()

    def wrapper_make_request(self, endpoint_name: str=None, params: dict={}) ->str :

        #Fetch the actual endpoint url to use
        correct_endpoint = self._fetch_endpoint(endpoint_name=endpoint_name)

        #Make call the the API for response
        response = self.make_request(endpoint=correct_endpoint, params=params)

        return response


import pytest
from unittest.mock import Mock

from OireachtasAPI import api, errors


@pytest.mark.parametrize(
    "endpoint",
    [
        "https://api.oireachtas.ie/v1/legislation",
        "https://api.oireachtas.ie/v1/debates",
        "https://api.oireachtas.ie/v1/constituencies",
        "https://api.oireachtas.ie/v1/parties",
        "https://api.oireachtas.ie/v1/divisions",
        "https://api.oireachtas.ie/v1/questions",
        "https://api.oireachtas.ie/v1/houses",
        "https://api.oireachtas.ie/v1/members",
    ],
)
def test_make_request_success(monkeypatch, response_factory, endpoint):
    mock_response = response_factory(status_code=200)
    mock_get = Mock(return_value=mock_response)
    monkeypatch.setattr(api.requests, "get", mock_get)

    response = api.API().make_request(endpoint=endpoint)

    assert response.status_code == 200
    mock_get.assert_called_once_with(url=endpoint, params={})


@pytest.mark.parametrize(
    ("status_code", "expected_exception"),
    [
        (400, errors.BadRequest),
        (401, errors.Unauthorised),
        (403, errors.Forbidden),
        (404, errors.NotFound),
        (429, errors.TooManyRequests),
    ],
)
def test_make_request_raises_for_error_status(monkeypatch, response_factory, status_code, expected_exception):
    mock_response = response_factory(status_code=status_code)
    mock_get = Mock(return_value=mock_response)
    monkeypatch.setattr(api.requests, "get", mock_get)

    with pytest.raises(expected_exception):
        api.API().make_request(endpoint="https://api.oireachtas.ie/v1/legislation")

    mock_get.assert_called_once()



"""Public package interface and logging helpers for the Oireachtas API wrapper."""

import logging
from typing import Optional

from .api import API as API
from .wrapper import Wrapper as Wrapper


DEFAULT_LOG_FORMAT = '%(asctime)s - %(name)s - %(levelname)s - %(message)s'


def configure_logging(
    *,
    level: int = logging.DEBUG,
    filename: Optional[str] = 'OireachtasWrapper_API.log',
    fmt: str = DEFAULT_LOG_FORMAT,
) -> None:
    """Opt-in helper to configure logging for users of the wrapper.

    Parameters mirror :func:`logging.basicConfig` while providing sensible
    defaults for consumers that want file-based logging for API interactions.
    """

    logging.basicConfig(filename=filename, level=level, format=fmt)



import pytest
from unittest.mock import Mock

from OireachtasAPI import api


@pytest.mark.parametrize(
    ("endpoint", "count_key"),
    [
        ("https://api.oireachtas.ie/v1/legislation", "billCount"),
        ("https://api.oireachtas.ie/v1/questions", "questionCount"),
        ("https://api.oireachtas.ie/v1/debates", "debateCount"),
        ("https://api.oireachtas.ie/v1/constituencies", "constituencyCount"),
        ("https://api.oireachtas.ie/v1/parties", "partyCount"),
    ],
)
def test_fetch_data_response_contains_expected_counts(monkeypatch, response_factory, endpoint, count_key):
    payload = {"head": {"counts": {count_key: 1}}}
    mock_response = response_factory(status_code=200, json_data=payload)
    mock_get = Mock(return_value=mock_response)
    monkeypatch.setattr(api.requests, "get", mock_get)

    response = api.API().make_request(endpoint=endpoint, params={"limit": 1})

    assert count_key in response.json()["head"]["counts"]
    mock_get.assert_called_once_with(url=endpoint, params={"limit": 1})



import pytest

from OireachtasAPI import urls


@pytest.mark.parametrize(
    ("method_name", "expected"),
    [
        ("_legislation_url", "https://api.oireachtas.ie/v1/legislation"),
        ("_debates_url", "https://api.oireachtas.ie/v1/debates"),
        ("_constituencies_url", "https://api.oireachtas.ie/v1/constituencies"),
        ("_parties_url", "https://api.oireachtas.ie/v1/parties"),
        ("_divisions_url", "https://api.oireachtas.ie/v1/divisions"),
        ("_questions_url", "https://api.oireachtas.ie/v1/questions"),
        ("_houses_url", "https://api.oireachtas.ie/v1/houses"),
        ("_members_url", "https://api.oireachtas.ie/v1/members"),
    ],
)
def test_url_builders_return_expected_values(method_name, expected):
    url_builder = urls.URLs()
    method = getattr(url_builder, method_name)
    assert method() == expected





class URLs:

    def __init__(self):
        self.base_url = 'https://api.oireachtas.ie/v1'

        #endpoints
        self.legislation = "/legislation"
        self.debates = '/debates'
        self.constituencies = '/constituencies'
        self.parties = '/parties'
        self.divisions = '/divisions'
        self.questions = '/questions'
        self.houses = '/houses'
        self.members = '/members'

    """Merges all the base urls with their respecitve endpoints and returns them
    All of these are hidden methods as to not pollute the namespace"""
    def _base_url(self):
        return self.base_url

    def _legislation_url(self):
        return self.base_url + self.legislation

    def _debates_url(self):
        return self.base_url + self.debates

    def _constituencies_url(self):
        return self.base_url + self.constituencies

    def _parties_url(self):
        return self.base_url + self.parties

    def _divisions_url(self):
        return self.base_url + self.divisions

    def _questions_url(self):
        return self.base_url + self.questions

    def _houses_url(self):
        return self.base_url + self.houses

    def _members_url(self):
        return self.base_url + self.members



import pytest
from unittest.mock import Mock

from OireachtasAPI import wrapper


@pytest.mark.parametrize(
    ("endpoint_name", "expected_url"),
    [
        ("legislation", "https://api.oireachtas.ie/v1/legislation"),
        ("debates", "https://api.oireachtas.ie/v1/debates"),
        ("constituencies", "https://api.oireachtas.ie/v1/constituencies"),
        ("parties", "https://api.oireachtas.ie/v1/parties"),
        ("divisions", "https://api.oireachtas.ie/v1/divisions"),
        ("questions", "https://api.oireachtas.ie/v1/questions"),
        ("houses", "https://api.oireachtas.ie/v1/houses"),
        ("members", "https://api.oireachtas.ie/v1/members"),
    ],
)
def test_fetch_endpoint_returns_expected_url(endpoint_name, expected_url):
    assert wrapper.Wrapper()._fetch_endpoint(endpoint_name=endpoint_name) == expected_url


def test_wrapper_make_request_uses_api_client(monkeypatch, response_factory):
    payload = {"head": {"counts": {"billCount": 1}}}
    mock_response = response_factory(status_code=200, json_data=payload)
    mock_make_request = Mock(return_value=mock_response)
    monkeypatch.setattr(wrapper.Wrapper, "make_request", mock_make_request)

    response = wrapper.Wrapper().wrapper_make_request(endpoint_name="legislation", params={"limit": 1})

    assert response.json()["head"]["counts"]["billCount"] == 1
    mock_make_request.assert_called_once_with(
        endpoint="https://api.oireachtas.ie/v1/legislation", params={"limit": 1}
    )



from setuptools import setup, find_packages

setup(
    name='OireachtasAPIWrapper',
    version='0.0.2',
    packages=find_packages(),
    url='https://github.com/aaronbowman/OireachtasAPIWrapper',
    license='MIT',
    author='Aaron Bowman',
    author_email='aaronrbowman12@gmail.com',
    description='Python client for the Irish Oireachtas public API',
    long_description=open('README.md', encoding='utf-8').read(),
    long_description_content_type='text/markdown',
    install_requires=[
        'requests>=2.31',
    ],
    classifiers=[
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
)



class OireachtasAPIException(Exception):
    """Baseline exception class for the Oireachtas API"""
    pass

class HTTPException(OireachtasAPIException):

    # Created with the defaults of none so that we can later cover json and HTTPS exceptions in the one area
    def __init__(self, response=None, response_json=None):
        self.response = response

        self.api_error = []
        self.api_code = []
        self.api_message = []

        try:
            status_code = response.status.code
        except AttributeError:
            # Response is an instance of http.clientresponse
            status_code = response.status

class BadRequest(HTTPException):
    """BadRequest()
    Exception raised for a 400 HTTP status code
    """
    pass

class Unauthorised(HTTPException):
    """Unauthorised()
    Exception raise for a 401 HTTP status code
    """
    pass

class Forbidden(HTTPException):
    """Forbidden()
    Exception raise for a 404 HTTP status code
    """
    pass

class TooManyRequests(HTTPException):
    """TooManyRequests()
    Exception raised for a 429 HTTP status code
    """
    pass

class NotFound(HTTPException):
    """NotFound()
    Exceotion riasaed for a 404 HTTP status code
    """
    pass